;; ============ AutoNum.lsp ===============
;;
;;  FUNCTION: Automatic Numbering
;;  Will sequentially place numerical
;;  at a specified Spacing, with optional
;;  Prefix and/or Suffix.
;;
;;  SYNTAX: AUTONUM
;;
;;  AUTHOR:
;;  Copyright (c) June 2009, Lee McDonnell
;; (Contact Lee Mac, CADTutor.net, TheSwamp.org)
;;
;;  ~ With Additional Thanks to:
;;     Charles Alan Butler (CAB)
;;
;;  PLATFORMS:
;;  Tested in ACAD 2004, ACAD 2010.
;;
;;  VERSION:
;;  1.0  ~  18.06.2009  ~ First Release
;;  2.0  ~  25.06.2009  ~ Changed Direction Method
;;  3.0  ~  25.06.2009  ~ Pick Angle added
;; ==========================================
;;
;;  4.0  ~  26.06.2009  ~ CAB combined routines, NumInc.lsp
;;                        A button for each function
;;
;; ==========================================
;;
;; ============ NumInc.lsp ===============
;;
;;  FUNCTION: Number Increment
;;  Will sequentially place numerical
;;  text upon mouse click, with optional
;;  prefix and suffix.
;;
;;  >>  Enter          =  Exit
;;  >>  Space          =  Rotate ACW
;;  >>  Shift + Space  =  Rotate CW
;;  >>  Tab            =  Rotate 90
;;  >>  Shift + Tab    =  Mirror Rotation
;;  >>  C              =  Curve Aligned
;;  >>  R              =  Replace Text/Attribute
;;  >>  T              =  Toggle Counter
;;
;;  AUTHOR:
;;  Copyright (c) 2009, Lee McDonnell
;;  (Contact Lee Mac, CADTutor.net)
;;
;;  PLATFORMS:
;;  Requires Express Tools for full Functionality.
;;  Tested in ACAD 2004, 2010.
;;
;;  VERSION:
;;  1.0  ~  12.04.2009  ~ First Release
;;  2.0  ~  14.04.2009  ~ Incremental Prefix/Suffix
;;  3.0  ~  15.04.2009  ~ DCL Version
;;  4.0  ~  15.04.2009  ~ Replace Text Added
;;  5.0  ~  16.04.2009  ~ Leading Zeros Allowed
;;  6.0  ~  18.06.2009  ~ Total Program Upgrade
;;  7.0  ~  27.06.2009  ~ Counter Toggle/Bug Fixes
;;
;; =======================================

(defun c:AutoNum  (/ *error* vlst ovar dcTag doc jlist
                     bPt i tStr Strl  styl space
                     pref end inc thgt spc snum suff
                     pflag mflag sflag flag
                     ang_pick
                   )

  (vl-load-com)
  ;;  Set Defaults for Global Variables
  (or num:snum (setq num:snum "1"))
  (or num:ninc (setq num:ninc "1"))
  (or num:pref (setq num:pref ""))
  (or num:suff (setq num:suff ""))
  (or num:end  (setq num:end "10"))
  (or inc:set  (setq inc:set "sel_mid"))
  (or dir:set  (setq dir:set "90.0"))
  (or spc:set  (setq spc:set "1"))
  (or ang:set  (setq ang:set "auto_ang"))
  (or ove:set  (setq ove:set "0.0"))
  (or num:thgt (setq num:thgt (rtos (getvar "TEXTSIZE"))))
  (or just:def (setq just:def "9")) ; CAB Base Left default
  (or *Mac$Per*  (setq *Mac$Per* 0.0))
  (or *Mac$tOff* (setq *Mac$tOff* 1.))

  
  ;;===============================================
  ;;         L o c a l   F u n c t i o n s         
  ;;===============================================
  (defun *error* (msg)
    (if ovar (mapcar 'setvar vlst ovar))
    (if (and msg (not (wcmatch (strcase msg) "*BREAK,*CANCEL*,*EXIT*"))) ; CAB
      (princ (strcat "\n<< Error: " msg " >>")))
      ;;  null some Global variables
      (mapcar '(lambda (x)(set (read x) nil))
              '("ANG" "CANG" "CPT" "CTOGG" "CURMOD" "DATA" "DCTAG" 
              "DEC" "DIR" "END" "ENT" "GR" "GRDAT" "INC" "INDEC" "LANG" 
              "LAYERS" "MFLAG" "MSG" "OVAR" "PFLAG" "PREF" "PT" "REPFLAG" 
              "REPMOD" "ROT" "SFLAG" "SHIFT" "SNUM" "SPC" "STRL" "STYLES" 
              "SUFF" "THGT" "TOBJ" "VLST" "X" "|72" "|73"))
    (redraw) ; in case of osnaps
    (princ))



;;  CAB - revised this subroutine 
(defun glst  (/ x y z)
  (setq lay:def  (get_tile "text_lay")
        styl:def (get_tile "text_styl")
        just:def (get_tile "text_just"))
  (setq |72 (/ (vl-position (atoi just:def)
                 '(0 3 6 9 1 4 7 10 2 5 8 11)) 4))
  (setq |73 (/ (vl-position (atoi just:def)
                 '(9 10 11 6 7 8 3 4 5 0 1 2)) 3)))

(defun get_values  ( / )
  (setq num:pref (get_tile "pref_str")
        num:snum (get_tile "mid_str")
        num:suff (get_tile "suff_str")
        num:ninc (get_tile "inc_str")
        num:end  (get_tile "end_str")
        spc:set  (get_tile "spc_str")
        num:thgt (get_tile "text_size")
        dir:set  (get_tile "sel_dir"))
  (and (eq "" num:ninc) (setq num:ninc "1"))
  (and (eq "" num:thgt) (setq num:thgt (rtos (getvar "TEXTSIZE"))))
  (and (eq "" spc:set)  (setq spc:set "0.0"))
  (and (eq ang:set "man_ang") (setq ove:set (get_tile "txt_ang")))
  (if (setq inDec (vl-string-position 46 num:ninc))
    (setq inDec (- (strlen num:ninc) inDec 1))
    (setq inDec 0)))

  ;;-------------------------------------------
  ;;vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
(defun errchk (eFlag / 
               Getlen txt2num )

  (defun Getlen (txt / decpos decs)
    (setq Strl (strlen txt))
    (if (setq decpos (vl-string-position 46 txt))
      (setq decs (- Strl decpos 1))
      (setq decs 0))
    (setq dec (max inDec decs))
    (cond ((and (> decs 0) (> inDec 0))
           (setq Strl (+ decpos (max decs inDec) 1)))
          ((and (zerop decs) (> inDec 0))
           (setq Strl (+ inDec 1 Strl)))))
  
  ;CAB 
  (defun txt2num  (txt)  
    (cond ((distof txt 5))
          ((distof txt 2))
          ((distof txt 1))
          ((distof txt 4))
          ((distof txt 3))
          (T nil)))

  ;;  ignore some checks when not in Array Mode
  (cond ((and (= eFlag 3)(not (setq dir (txt2num dir:set))))
         (set_tile "error" "Direction Angle not Numerical!"))
        ((and (eq "man_ang" ang:set) (not (setq ang (txt2num ove:set))))
         (set_tile "error" "Angle Override not Numerical!"))
        ((and (eq "sel_pref" inc:set) (not (setq pref (txt2num num:pref))))
         (set_tile "error" "Prefix not Numerical!"))
        ((and (eq "sel_mid" inc:set) (not (setq snum (txt2num num:snum))))
         (set_tile "error" "Middle not Numerical!"))
        ((and (eq "sel_suff" inc:set) (not (setq suff (txt2num num:suff))))
         (set_tile "error" "Suffix not Numerical!"))
        ((and (= eFlag 3)(not (setq end (txt2num num:end))))
         (set_tile "error" "End Number not Numerical!"))
        ((not (setq inc (txt2num num:ninc)))
         (set_tile "error" "Increment not Numerical!"))
        ((not (setq thgt (txt2num num:thgt)))
         (set_tile "error" "Text Height not Numerical!"))
        ((and (= eFlag 3)(not (setq spc (txt2num spc:set))))
         (set_tile "error" "Spacing not Numerical!"))
        ((and (= eFlag 3)
              (or (and pref
                  (or (and (minusp inc) (< pref end))
                      (and (not (minusp inc)) (< end pref))))
             (and snum
                  (or (and (minusp inc) (< snum end))
                      (and (not (minusp inc)) (< end snum))))
             (and suff
                  (or (and (minusp inc) (< snum end))
                      (and (not (minusp inc)) (< end snum))))))                
         (set_tile "error" "Error in Increment and End Number Combination!"))
        ((zerop inc) (set_tile "error" "Increment Cannot be Zero!") )

        (t  ;  No Error
         (cond
           ((eq "sel_pref" inc:set) (setq pflag T) (GetLen num:pref))
           ((eq "sel_mid"  inc:set) (setq mflag T) (GetLen num:snum))
           ((eq "sel_suff" inc:set) (setq sflag T) (GetLen num:suff)))
         (done_dialog eFlag))))
  ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^
  ;;-------------------------------------------

  ;;  Add Text to Drawing
(defun txt (pt str rot)
  (entmakex
    (list
      (cons 0 "TEXT")
      (cons 8 (nth (atoi lay:def) layers))
      (cons 10 pt)
      (cons 40 thgt)
      (cons 1 str)
      (cons 50 rot)
      (cons 7  (nth (atoi styl:def) styles))
      (cons 71 0)
      (cons 72 |72)
      (cons 73 |73)
      (cons 11 pt))))

(defun ang_pick (/ ang)
  (if (setq ang (getangle "\nPick Angle: "))
    (setq dir:set (rtos (* 180. (/ ang pi)) 2 1))
    (setq dir:set "0.0")))

(defun nest_info  (dcTagi)
  (or (new_dialog "AutoInfo" dcTagi)
      (alert "Error Loading Information.")
      (exit))
  (action_tile "accept" "(done_dialog)")
  (start_dialog))

  ;;  Return the Alignment Code
  (defun get_vla-alignment (idx)
   ;; *vla Alignment codes*   dText
   (eval
     (nth (atoi idx)
     '(acAlignmentTopLeft 
       acAlignmentTopCenter 
       acAlignmentTopRight
       acAlignmentMiddleLeft 
       acAlignmentMiddleCenter 
       acAlignmentMiddleRight 
       acAlignmentBottomLeft 
       acAlignmentBottomCenter 
       acAlignmentBottomRight
       acAlignmentLeft 
       acAlignmentCenter 
       acAlignmentRight 
       )))
  )

  ;;  Increment num with ival
  ;;  Pad out String as needed
  (defun PrepareNum (num$ num ival)
     (set num$ (rtos (set num (+ (eval num) ival)) 2 dec))
               (while (< (strlen (eval num$)) Strl)
                 (set num$ (strcat (chr 48) (eval num$))))
  )

  ;; Object Snap functions for GrRead Loops:

  (defun oSlst (os / str cnt)
  (setq str "" cnt 0)
  (if (< 0 os 16383)
    (foreach mod '("_end" "_mid" "_cen" "_nod" "_qua"
                   "_int" "_ins" "_per" "_tan" "_nea"
                   "_non" "_app" "_ext" "_par")
      (if (not (zerop (logand (expt 2 cnt) os)))
        (setq str (strcat str mod (chr 44))))
      (setq cnt (1+ cnt))))
  (vl-string-right-trim (chr 44) str))

(defun osMark (pt / drft osSz osCol ratio bold glst i)
  (setq drft (vla-get-drafting
               (vla-get-preferences
                 (vlax-get-acad-object)))
        osSz (vla-get-AutoSnapMarkerSize drft)
        oscol (vla-get-AutoSnapMarkerColor drft)
        ratio (/ (getvar "VIEWSIZE")
               (cadr (getvar "SCREENSIZE")))
        bold (mapcar
               (function
                 (lambda (x)
                   (* x ratio))) (list (+ osSz 0.5) osSz (- osSz 0.5))) i 0)

  (repeat 50
    (setq glst
      (cons
        (polar '(0 0 0) (* i (/ pi 25.)) 1.) glst) i (1+ i)))      

  (foreach x bold
     (grvecs (append (list oscol) glst (cdr glst) (list (car glst)))
             (list (list  x  0.0 0.0 (car pt))
                   (list 0.0  x  0.0 (cadr pt))
                   (list 0.0 0.0 1.0 0.0)
                   (list 0.0 0.0 0.0 1.0))))) 

  
  
;; o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o
;;                M A I N   R O U T I N E                       
;; o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o

  (setq vlst '("DIMZIN" "MODEMACRO" "CMDECHO") ; CAB added CMDECHO
        ovar (mapcar 'getvar vlst))
  (mapcar 'setvar vlst
    (list 3 (strcat "Rotation: 0.00" (chr 186)) 0))
  
    ;;  CAB moved this code out of the loop
    ;;VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
    (setq doc (vla-get-ActiveDocument (vlax-get-acad-object))
          space (if (zerop (vla-get-activespace doc))
                (if (= (vla-get-mspace doc) :vlax-true)
                  (vla-get-modelspace doc) ; Vport
                  (vla-get-paperspace doc))
                (vla-get-modelspace doc)))
    
    ;;  Pick List for Layers
    (vlax-for lay (vla-get-layers doc)
      (setq layers (cons (vla-get-Name lay) layers)))
    (setq layers (acad_strlsort layers)) ; CAB sort
    ;;  Pick List for Text Styles
    (vlax-for styl (vla-get-TextStyles doc)
      (setq styles (cons (vla-get-Name styl) styles)))
    (setq styles (vl-remove-if
                   (function
                     (lambda (x)
                       (eq "" x))) (acad_strlsort styles))) ; CAB sort

    ;;  load defaults for first run, Global after that
    (or lay:def  (setq lay:def (itoa (vl-position (getvar "cLayer") layers))))  ; CAB
    (or styl:def (setq styl:def (itoa (vl-position (getvar "textstyle") styles)))) ; CAB

    ;;  Pick List for Text Justification
    (setq jlist '("Top-Left"    "Top-Center"    "Top-Right"
                  "Middle-Left" "Middle-Center" "Middle-Right"
                  "Bottom-Left" "Bottom-Center" "Bottom-Right"
                  "Base-Left"   "Base-Center"   "Base-Right"))
  
  (setq dcTag (load_dialog "AutoNum V4.dcl"))
  
  ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

  ;;  Begin DCL Loop
  (while (not (vl-position flag '(0 1 3 4)))

    (or (new_dialog "AutoNum" dcTag)
        (alert "Error Loading AutoNumber.")
        (exit))

    (start_list "text_lay" 3)
    (mapcar 'add_list layers)
    (end_list)

    (start_list "text_styl" 3)
    (mapcar 'add_list styles)
    (end_list)

    (start_list "text_just" 3)
    (mapcar 'add_list jlist)
    (end_list)

    (mapcar 'set_tile ; set DCL Tiles
      '("pref_str" "mid_str" "suff_str" "inc_str" "end_str" "spc_str" "text_size" "sel_dir" "txt_ang")
      (list num:pref  num:snum  num:suff   num:ninc  num:end   spc:set   num:thgt     dir:set  ove:set))

    (set_tile inc:set "1")
    (set_tile ang:set "1")
    (and lay:def  (set_tile "text_lay" lay:def))
    (and styl:def (set_tile "text_styl" styl:def))
    (and just:def (set_tile "text_just" just:def))
    (mode_tile "txt_ang" (if (eq ang:set "auto_ang") 1 0))

    (action_tile "sel_pref"  "(setq inc:set \"sel_pref\")")
    (action_tile "sel_mid"   "(setq inc:set \"sel_mid\")")
    (action_tile "sel_suff"  "(setq inc:set \"sel_suff\")")
    (action_tile "auto_ang"  "(setq ang:set \"auto_ang\") (mode_tile \"txt_ang\" 1)")
    (action_tile "man_ang"   "(setq ang:set \"man_ang\")  (mode_tile \"txt_ang\" 0)")
    (action_tile "pik_ang"   "(get_values) (done_dialog 2)")
    (action_tile "addarray"  "(get_values) (glst) (errchk 3)")
    (action_tile "accept"    "(get_values) (glst) (errchk 4)")
    (action_tile "help"      "(AutoNumHelp)")
    (action_tile "info"      "(nest_info dcTag)")
    (action_tile "cancel"    "(done_dialog)")

    (setq flag (start_dialog))
    (if (eq flag 2) (ang_pick)))
    ;;  End DCL Loop
  
    (unload_dialog dcTag) ; CAB moved this out of the loop
  
  ;;  Insert Text via an Array
  (if (and (= flag 3) (or pflag mflag sflag)
           (setq bPt (getpoint "\nSelect Base Point: ")))
    (progn
      (command "._undo" "_Begin") ; CAB
      (setq i 0 num:end
        (vl-princ-to-string
          (+ end inc (- end (cond (pflag pref) (mflag snum) (sflag suff))))))
      (and (eq "auto_ang" ang:set) (setq ang (- dir 90.)))
      (cond (pflag
             (setq num:pref (rtos (distof num:pref) 2 dec))
             (while (< (strlen num:pref) Strl)
               (setq num:pref (strcat (chr 48) num:pref))))
            (mflag
             (setq num:snum (rtos (distof num:snum) 2 dec))
             (while (< (strlen num:snum) Strl)
               (setq num:snum (strcat (chr 48) num:snum))))
            (sflag
             (setq num:suff (rtos (distof num:suff) 2 dec))
             (while (< (strlen num:suff) Strl)
               (setq num:suff (strcat (chr 48) num:suff)))))
      (while ((if (minusp inc) <= >=) end
               (cond (pflag pref)
                     (mflag snum)
                     (sflag suff)))
        (setq tstr (strcat num:pref num:snum num:suff))
        ;;  Add Text to Drawing
        (txt (polar bPt (* pi (/ dir 180.)) (* i spc)) tStr (* pi (/ ang 180.)))
        
          (cond
            (pflag (PrepareNum 'num:pref 'pref inc))
            (mflag (PrepareNum 'num:snum 'snum inc))
            (sflag (PrepareNum 'num:suff 'suff inc)))
        
        (setq i (1+ i)))
      (command "._undo" "_End") ; CAB
      ))

    ;;----------------------------------------------------
    ;;   Insert Text one by one, previously NumInc.lsp
    (if (and (= flag 4) (or pflag mflag sflag))
      (progn
        
  (if (vl-catch-all-error-p
        (vl-catch-all-apply
          '(lambda () (acet-sys-shift-down))))
    (alert "\n** ExpressTools not loaded. Some Functions May not be Available. **")
    (setq shift T)) ; Express Tools Marker
      
      (cond (pflag (PrepareNum 'num:pref 'pref 0))
            (mflag (PrepareNum 'num:snum 'snum 0))
            (sflag (PrepareNum 'num:suff 'suff 0)))
  
      (setq rot  0.0
            flag nil)
      (princ (setq msg "\n<<Exit>> [C]urve Aligned, [Space] = Rotate ACW, [Tab] = Rotate 90 [T]oggle Count
                        \n[Shift] + [Space] = Rotate CW, [Shift] + [Tab] = Mirror Rotation, [R]eplace"))
      (while (not flag)
        (setq tstr (strcat num:pref num:snum num:suff))
        (if (not RepMod) ; create a new Text Object
          (progn
            (setq tObj (vla-addText space tStr
                         (vlax-3D-point '(0 0 0)) tHgt))
            (vla-put-layer tObj (nth (atoi Lay:def) layers))
            (vla-put-StyleName tObj (nth (atoi styl:def) styles))
            (vla-put-Alignment tObj (get_vla-alignment just:def)) ; CAB
            (vla-put-visible tObj :vlax-false)))           
        (while
          (progn
            (setq grdat (grread t 15 0)
                  data  (cadr grdat))
            (setvar "MODEMACRO"
              (strcat "Rotation: "
                (rtos (rem (fix (* 180. (/ rot pi))) 360) 2 2) (chr 186)))
            (cond
              ((and (= 5 (car grdat)) (listp data))
               (and (not RepMod) (vla-put-visible tObj :vlax-true))
               (redraw)
               (cond
                 (CurMod ; Curve Aligned
                  (vla-put-visible tObj :vlax-false)
                  (if (and (setq ent (car (entsel "\nSelect Curve to Align <Exit>: ")))
                           (vl-position (cdr (assoc 0 (entget ent)))
                             '("LINE""ARC""CIRCLE""ELLIPSE""LWPOLYLINE""POLYLINE""REGION""SPLINE")))
                    (progn
                      (vla-put-visible tObj :vlax-true)
                      (princ "\n<< Type [+] or [-] for offset, and [P]erpendicular >>")
                      (while (or (and (setq gr (grread t 15 0)) (eq (car gr) 5))
                                 (and (eq 2 (car gr)) (vl-position (cadr gr) '(43 45 61 80 112))))
                        (cond ((and (eq 5 (car gr)) (listp (setq cPt (cadr gr))))
                               (setq pt (vlax-curve-getClosestPointto ent cPt)
                                     cAng (angle pt cPt)
                                     lAng (+ cAng *Mac$Per*))
                               (cond ((and (> lAng (/ pi 2)) (<= lAng pi))
                                      (setq lAng (- lAng pi)))
                                     ((and (> lAng pi) (<= lAng (/ (* 3 pi) 2)))
                                      (setq lAng (+ lAng pi))))
                               (vla-move tObj
                                 (if (= Just:Def "9")
                                   (vla-get-InsertionPoint tObj)
                                   (vla-get-TextAlignmentPoint tObj)
                                 )
                                   (vlax-3D-point (polar pt cAng (* tHgt *Mac$tOff*))))
                               (vla-put-Rotation tObj lAng))
                              ((eq 2 (car gr))
                               (cond ((vl-position (cadr gr) '(43 61))
                                      (setq *Mac$tOff* (+ (/ 1 10.) *Mac$tOff*)))
                                     ((eq (cadr gr) 45)
                                      (setq *Mac$tOff* (-  *Mac$tOff* (/ 1 10.))))
                                     ((vl-position (cadr gr) '(80 112))
                                      (setq *Mac$Per* (- (/ pi 2.) *Mac$Per*))))))) nil) ; Exit Loop
                    (progn
                      (vla-put-visible tObj :vlax-true)
                      (setq CurMod nil) (princ msg))))
                 (RepMod ; Replacement Mode
                  (and tObj (not (vlax-erased-p tObj))
                       (vla-delete tObj))
                  (if (and (setq ent (car (nentsel "\nSelect Text, MText or Attribute << Hit Enter to Exit >>: ")))
                           (wcmatch (cdr (assoc 0 (entget ent))) "*TEXT,ATTRIB*"))
                    (progn
                      (entmod (subst (cons 1 tstr) (assoc 1 (entget ent)) (entget ent)))
                      (vla-regen doc acActiveViewport) nil) ; Exit Loop
                    (progn
                      (setq RepMod nil Repflag T) (princ msg) nil))) ; Exit Loop
                 (t ; Else, Manual Placement
                  (if (and (< 0 (getvar "OSMODE") 16383)
                           (setq osPt (osnap data (osLst (getvar "OSMODE")))))
                    (progn
                      (osMark osPt) (setq data osPt)))
                  (vla-move tObj (if (= Just:Def "9")
                                   (vla-get-InsertionPoint tObj)
                                   (vla-get-TextAlignmentPoint tObj)
                                 )
                            (vlax-3D-point data))
                  (vla-put-rotation tObj rot) t))) ; Stay in Loop
              
              ((and (= 2 (car grdat)) (= 32 data))
               (if (and shift (acet-sys-shift-down))
                 (setq rot (- rot (/ pi 180.0)))
                 (setq rot (+ (/ pi 180.0) rot))) t)
              ((and (= 2 (car grdat)) (= 9 data))
               (if (and shift (acet-sys-shift-down))
                 (setq rot (* -1.0 rot))
                 (progn
                   (if (not (vl-position rot (list 0.0 (/ pi 2) pi (/ (* 3 pi) 2))))
                     (setq rot 0.0)
                     (setq rot (+ (/ pi 2) rot)))))t)
              ((and (= 2 (car grdat)) (eq 6 data))
               (cond ((< 0 (getvar "OSMODE") 16384)
                      (setvar "OSMODE" (+ 16384 (getvar "OSMODE"))))
                     (t (setvar "OSMODE" (- (getvar "OSMODE") 16384)))))
              ((and (= 2 (car grdat)) (vl-position data '(99 67))) ; C/c
               (setq CurMod T)t)
              ((and (= 2 (car grdat)) (vl-position data '(82 114))) ; R/r
               (setq RepMod T)t)
              ((and (= 2 (car grdat)) (vl-position data '(84 116))) ; T/t
               (setq CTogg (not CTogg)) t) ; Keep in Loop
              ((or (and (= 2 (car grdat)) (eq data 13)) ; Enter
                   (= 25 (car grdat))) ; Right Click
               (setq flag T)
               (and tObj (not (vlax-erased-p tObj))
                    (vla-delete tObj))
               ;;  Cancel the last Increment
        (if (and ;|(not RepFlag)|; (not cTogg)) ; CAB Replace Mode should also deIncrement
          (cond
            (pflag (PrepareNum 'num:pref 'pref (- inc)))
            (mflag (PrepareNum 'num:snum 'snum (- inc)))
            (sflag (PrepareNum 'num:suff 'suff (- inc)))))
               (princ "\n** User Quit **")nil) ; Exit Loop
              ((and (= 3 (car grdat)) (listp data)) ; Text Placed
               (if
                 (and
                   (setq tss
                     (ssget "_X" '((0 . "ACAD_TABLE"))))
                   (setq lst
                     (car
                       (vl-remove-if 'null
                         (mapcar
                           (function
                             (lambda (tab)
                               (if
                                 (eq :vlax-true
                                   (vla-HitTest tab
                                     (vlax-3D-point
                                       (trans data 1 0))
                                         (vlax-3D-point
                                           (trans
                                             (getvar 'VIEWDIR) 1 0)) 'row 'col))
                                 (list tab row col))))
                           (mapcar 'vlax-ename->vla-object
                             (mapcar 'cadr (ssnamex tss))))))))
                 (and
                   (not             
                     (apply 'vla-SetText
                       (append lst (list tStr)))) tObj
                     (not (vlax-erased-p tObj))
                   (vla-delete tObj)))
               nil) ; Exit Loop
              (t ; Key Unaccounted
               (princ "\n** Invalid KeyPress **")
               (princ msg) t)))) ; Stay in Loop
        
        (if (and (not RepFlag) (not cTogg))
          (cond
            (pflag (PrepareNum 'num:pref 'pref inc))
            (mflag (PrepareNum 'num:snum 'snum inc))
            (sflag (PrepareNum 'num:suff 'suff inc)))
           (setq Repflag nil)))
        
      )
    )

  ;;(mapcar 'setvar vlst ovar) ; CAB removed
  (*error* nil) ; clear variables ; CAB added
  (princ))
;;; -=<*>=- -=<*>=- -=<*>=- -=<*>=- -=<*>=- -=<*>=- -=<*>=- -=<*>=-

(defun AutoNumHelp()
(alert (strcat 
"***  Incremental Numbering Suite  *** "
"\nThis program enables the user to place incrementing numerical text in a drawing, with a range of positioning"
"\nutilities and an optional Prefix/Suffix."
"\n"
"\nThe program is invoked with the syntax: Autonum"
"\n"
"\nUpon invoking the program, a user interface will appear, with options to change text formatting, layers, and content."
"\nOne can enter Prefix, Middle and Suffix text and have the option to increment either section (providing it is numerical), "
"\nwith the ability to have leading zeros. One can also specify any increment (including decimals), and alter the text "
"\nlayer, style and height."
"\n"
"\nThe [Add Array] button will sequentially place an array of numerical text at a specified spacing, with an optional "
"\nPrefix and Suffix."
"\n"
"\nThe [Pick Each Insert] button will sequentially place numerical text at a user specified point upon mouse click, "
"\nwith optional Prefix and Suffix."
"\n"
"\n[Pick Each Insert] Options"
"\nUpon submitting the options, the user will see a preview of the text on screen, with various options displayed "
"\nat the command line."
"\n"
"\nFrom this interface, the user has various placement options, including:"
"\n    Space........... Rotates Text Anti-Clockwise. "
"\n    Shift+Space.. Rotates Text Clockwise. "
"\n    Tab.............. Set Rotation back to zero, and rotates by 90. "
"\n    Shift+Tab..... Mirrors Rotation. "
"\n    C or c........... Switches to Curve Aligned Mode. "
"\n    R or r........... Switches to Replacement Mode. "
"\n    T or t............Toggle the Incrementing Counter."
"\n"
"\n  >>>  Curve Aligned Mode  <<<"
"\n  Upon pressing \"C\" or \"c\", the user will be prompted to select a curve. The text will then follow that curve,"
"\n  until the user clicks to place the text. Text can be aligned both parallel and perpendicular to the curve,"
"\n  and options for additional refinement are displayed at the command line."
"\n  "
"\n  To exit this mode, just hit enter, or click empty space."
"\n  "
"\n  >>>  Replacement Mode  <<<"
"\n  Upon pressing \"R\" or \"r\", the user will be prompted to select either TEXT, MTEXT or ATTIButes, which,"
"\n  upon selection will be modified to display the incremental text."
"\n"
"\n  To exit this mode, either hit enter, or click empty space."
)))




(princ
  (vl-list->string
    '(10 42 42 32 65 117 116 111 78 117 109 98 101 114 32 76 111 97 100 101 100
      44 32 116 121 112 101 32 34 65 117 116 111 78 117 109 34 32 116 111 32 105
      110 118 111 107 101 32 42 42 10 32 32 32 45 32 67 111 112 121 114 105 103 
      104 116 32 169 32 74 117 110 101 32 50 48 48 57 44 32 76 101 101 32 77 99 
      68 111 110 110 101 108 108 32 45 32)))
(princ)